// Copyright (C) 2019  Wilko Manger
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

import 'package:meta/meta.dart';

import '../room_event.dart';
import '../../../identifier.dart';
import '../../event.dart';

import 'state_event.dart';

class AliasesChangeEvent extends StateEvent implements HasDiff {
  static const matrixType = 'm.room.aliases';

  @override
  final String type = matrixType;

  @override
  final AliasesChange content;

  @override
  final AliasesChange previousContent;

  @override
  final AliasesChangeDiff diff;

  List<Change<RoomAlias>> get changes =>
      diff.aliases.where((c) => c is! Unchanged);

  AliasesChangeEvent(
    RoomEventArgs args, {
    @required this.content,
    this.previousContent,
    @required String stateKey,
  })  : diff = AliasesChangeDiff(previousContent, content),
        super(args, stateKey: stateKey);
}

class AliasesChange extends EventContent {
  final List<RoomAlias> aliases;

  AliasesChange({@required this.aliases});

  @override
  List<Object> get props => [aliases];

  factory AliasesChange.fromJson(Map<String, dynamic> content) {
    if (content == null) return null;

    var aliases = content['aliases'] as List<dynamic>;

    List<RoomAlias> convertedAliases;
    if (aliases != null) {
      convertedAliases = aliases
          .map(
            (a) => RoomAlias.isValidFullyQualified(a) ? RoomAlias(a) : null,
          )
          .where((a) => a != null)
          .toList(growable: false);
    }

    return AliasesChange(
      aliases: convertedAliases,
    );
  }

  @override
  Map<String, dynamic> toJson() => super.toJson()
    ..addAll({
      'aliases': aliases?.map((a) => a.toString())?.toList(growable: false),
    });
}

class AliasesChangeDiff extends Diff {
  final AliasesChange _previous;
  final AliasesChange _current;

  AliasesChangeDiff(this._previous, this._current);

  @override
  List get props => [aliases];

  List<Change<RoomAlias>> get aliases =>
      Change.list(_previous?.aliases, _current.aliases);
}
