// Copyright (C) 2020  Wilko Manger
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

import '../../event/room/state/member_change_event.dart';

import '../../identifier.dart';
import '../../user.dart';

import 'membership.dart';

/// A user that's a member of a certain room.
class Member extends User {
  /// Event this member's state is based on.
  final MemberChangeEvent event;

  @override
  UserId get id => event.subjectId;

  RoomId get roomId => event.roomId;
  DateTime get since => event.time;

  Membership get membership => event.content.membership;

  @override
  Uri get avatarUrl => event.content.avatarUrl;

  @override
  String get name => event.content.displayName;

  Member.fromEvent(this.event);

  @override
  List<Object> get props => [event];
}
